import os

import boto3
import click
from pkg_resources import DistributionNotFound, get_distribution
from requests_toolbelt.sessions import BaseUrlSession

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    pass


class StumpfwaldSession(BaseUrlSession):
    page_size = 50

    def __init__(self, base_url):
        super(StumpfwaldSession, self).__init__(base_url=base_url)
        self.headers.update({
            'User-agent': 'stumpfwald/%s' % __version__,
            'Content-Type': 'application/json',
            'X-Stumpf-Token': os.getenv('X_STUMPF_TOKEN')
        })


def authorized(func):
    def authorized_func(*args, **kwargs):
        if 'X_STUMPF_TOKEN' not in os.environ:
            raise click.UsageError('authorization required: set variable X_STUMPF_TOKEN')
        func(*args, **kwargs)
    return authorized_func


def get_production_database_uri_secretsmanager():
    client = boto3.client('secretsmanager')
    username = client.get_secret_value(SecretId='stumpf/rds_readonly.username')['SecretString']
    password = client.get_secret_value(SecretId='stumpf/rds_readonly.password')['SecretString']
    host = client.get_secret_value(SecretId='stumpf/rds_readonly.host')['SecretString']
    dbname = client.get_secret_value(SecretId='stumpf/rds_readonly.dbname')['SecretString']
    return f'postgresql://{username}:{password}@{host}/{dbname}'


@click.group()
@click.option('--api-url', default='http://localhost:5000/api/v1/', envvar='STUMPF_API_URL')
@click.version_option()
@click.pass_context
def cli(ctx, api_url):
    session = StumpfwaldSession(api_url)
    ctx.obj = {
        'session': session
    }


from stumpfwald.app import app  # noqa: F401,E402
from stumpfwald.assignment import assignment  # noqa: F401,E402
from stumpfwald.task import task  # noqa: F401,E402
from stumpfwald.video import video  # noqa: F401,E402
