import json
import sys

import click

from stumpfwald import authorized, cli

DELAY = 2  # delay between posting tasks for AWS rate limiting


def valid_json(value):
    # https://github.com/pallets/click/issues/950
    try:
        if isinstance(value, dict):
            value = json.dumps(value)
        args = json.loads(value)
        return args
    except Exception:
        raise click.BadParameter('args should be valid json')


def generate_hit_object(type, video_id, app_id, args):
    args = valid_json(args)
    url = f'task/{video_id}/{app_id}/{type.lower()}'
    cost = 0
    if type == 'HIT':
        cost = args['reward'] * args['max_assignments']
    return {
        'url': url,
        'args': args,
        'video_id': video_id,
        'app_id': app_id,
        'cost': cost
    }


@click.group(name='task', short_help='The task resource.')
@click.pass_obj
def task(ctx):
    pass


@task.command(name='generate', help='Generate manifest file for task(s)')
@click.argument('type', type=click.Choice(['HIT', 'KWIT']))
@click.argument('video_id')
@click.argument('app_id')
@click.argument('args', default='{}')
@click.option('-o', '--output', type=click.File(mode='a'), default=sys.stdout)
@click.pass_obj
@authorized
def generate(ctx, type, video_id, app_id, args, output):
    output.write(json.dumps(generate_hit_object(type, video_id, app_id, args)) + '\n')


@task.command(name='submit', help='Submit generated task(s)')
@click.argument('input', type=click.File(mode='r'), nargs=-1)
@click.pass_obj
@authorized
def submit(ctx, input):

    def request(source, finished, failed):
        for line in source:
            task = json.loads(line.strip('\n'))
            r = ctx['session'].post(task['url'], json=task['args'])
            if not r.ok:  # Not reliably json
                failed.write(r.text + '\n')
                failed.flush()
            else:
                finished.write(r.text + '\n')
                finished.flush()

    if len(input) == 0:
        request(sys.stdin, sys.stdout, sys.stderr)
    else:
        for source in input:
            with open(f'{source.name}_finished.jsonl', 'w') as finished, \
                    open(f'{source.name}_failed.jsonl', 'w') as failed:
                request(source, finished, failed)


@task.command(name='details', help='Get Details about a task')
@click.argument('task_ids', nargs=-1)
@click.pass_obj
@authorized
def details(ctx, task_ids):
    assignments = []
    for task_id in task_ids:
        r = ctx['session'].get(f'task/{task_id}/assignments')
        if not r.ok:
            click.echo(click.style(r.text + '\n', fg='red'))
        else:
            assignments.extend(r.json())
    click.echo(json.dumps(assignments, indent=4))


@task.command(name='list', help='List tasks')
@click.pass_obj
@authorized
def list_(ctx):
    r = ctx['session'].get('task')

    if not r.ok:
        click.echo(r.text, err=True)
    else:
        click.echo(json.dumps(r.json(), indent=4))


cli.add_command(task)
